import R from 'ramda';
import React, {Component} from 'react';
import moment from 'moment';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import KitchenTestComponent from '../playground/kitchen-test-component';
import {XlReactDipBoolean} from '../../xl-react-dip/xl-react-dip-boolean/xl-react-dip-boolean';
import {XlReactDipCi} from '../../xl-react-dip/xl-react-dip-ci/xl-react-dip-ci';
import {XlReactDipDatetime} from '../../xl-react-dip/xl-react-dip-datetime/xl-react-dip-datetime';
import {XlReactDipEnum} from '../../xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';
import {XlReactDipInteger} from '../../xl-react-dip/xl-react-dip-integer/xl-react-dip-integer';
import {XlReactDipString} from '../../xl-react-dip/xl-react-dip-string/xl-react-dip-string';
import {XlReactDipList} from '../../xl-react-dip/xl-react-dip-list/xl-react-dip-list';
import {XlReactDipListOfCi} from '../../xl-react-dip/xl-react-dip-list-of-ci/xl-react-dip-list-of-ci';
import {XlReactDipSetOfCi} from '../../xl-react-dip/xl-react-dip-set-of-ci/xl-react-dip-set-of-ci';
import {XlReactDipSetOfStrings} from '../../xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings';
import {XlReactDipMapStringString} from '../../xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string'; // eslint-disable-line max-len
import {autocompleteHandlers, getHandlers} from '../handlers/predefined-handlers';
import {WARNING} from '../../utils/ci-utils';
import {dipInitialState} from './dip-initial-state';

export class Dip extends Component {
    static propTypes = {match: PropTypes.object};
    state = dipInitialState;

    render() {
        const {match} = this.props;

        /* eslint-disable sort-keys */
        const dateTimeModel = moment({
            year: 2016, month: 10, day: 9, hours: 15, minutes: 10, seconds: 0, milliseconds: 0
        }).toDate();
        /* eslint-enable sort-keys */

        const warningValidator = () => ({
            condition: true,
            message: {show: false, text: 'Sample Warning', type: WARNING}
        });

        const dataSetOfCi = {
            data: [
                {id: 'Item 1', type: '10'},
                {id: 'Item 2', type: '10'},
                {id: 'Item 3', type: '10'},
                {id: 'Item 3', type: '10'},
                {id: 'Item 4', type: '10'}
            ]
        };

        const updateState = (modelName, newState) => {
            const updatedState = {...this.state[modelName], ...newState};
            this.setState({...this.state, [modelName]: {...updatedState}});
            return updatedState;
        };

        const dips = {
            dipArrayOfStrings1: <KitchenTestComponent
                className="xl-react-dip-list-without-auto-complete"
                currentState={this.state}
                key={'dipArrayOfStrings1'}
                label={'Without autocomplete handler'}
                modelName={'dipArrayOfStrings1'}
                updateState={updateState}>
                <XlReactDipList/>
            </KitchenTestComponent>,
            dipArrayOfStrings2: <KitchenTestComponent
                className="xl-react-dip-list-with-auto-complete"
                currentState={this.state}
                key={'dipArrayOfStrings2'}
                label={'With autocomplete handler'}
                modelName={'dipArrayOfStrings2'}
                updateState={updateState}>
                <XlReactDipList
                    handlers={{NameList4: getHandlers()}}
                    objLabel={'id'}
                />
            </KitchenTestComponent>,
            dipBoolean1: <KitchenTestComponent
                className="xl-react-dip-boolean"
                currentState={this.state}
                key={'dipBoolean1'}
                modelName={'dipBoolean1'}
                updateState={updateState}>
                <XlReactDipBoolean/>
            </KitchenTestComponent>,
            dipCi1: <KitchenTestComponent
                className="xl-react-dip-ci-initial"
                currentState={this.state}
                key={'dipCi1'}
                label={'CI'}
                modelName={'dipCi1'}
                updateState={updateState}>
                <XlReactDipCi handlers={autocompleteHandlers} objLabel="id"/>
            </KitchenTestComponent>,
            dipCi2: <KitchenTestComponent
                className="xl-react-dip-ci-with-default-value"
                currentState={this.state}
                key={'dipCi2'}
                label={'CI with given value'}
                modelName={'dipCi2'}
                updateState={updateState}>
                <XlReactDipCi handlers={autocompleteHandlers} objLabel="id"/>
            </KitchenTestComponent>,
            dipDateTime1: <KitchenTestComponent
                className={'xl-react-dip-datetime-component'}
                currentState={this.state}
                key={'dipDateTime1'}
                label={'Date and time'}
                modelName={'dipDateTime1'}
                updateState={updateState}>
                <XlReactDipDatetime model={dateTimeModel}/>
            </KitchenTestComponent>,
            dipEnum1: <KitchenTestComponent
                className="xl-react-dip-enum-initial"
                currentState={this.state}
                key={'dipEnum1'}
                label={'Enumeration of strings'}
                modelName={'dipEnum1'}
                updateState={updateState}>
                <XlReactDipEnum/>
            </KitchenTestComponent>,
            dipInteger1: <KitchenTestComponent
                className="xl-react-dip-integer-default"
                currentState={this.state}
                key={'dipInteger1'}
                label={'Without default value'}
                modelName={'dipInteger1'}
                updateState={updateState}>
                <XlReactDipInteger validators={[warningValidator]}/>
            </KitchenTestComponent>,
            dipInteger2: <KitchenTestComponent
                className="xl-react-dip-integer-with-default-value"
                currentState={this.state}
                key={'dipInteger2'}
                label={'With default value'}
                modelName={'dipInteger2'}
                updateState={updateState}>
                <XlReactDipInteger/>
            </KitchenTestComponent>,
            dipListOfCi1: <KitchenTestComponent
                className={'dip-list-of-ci-component'}
                currentState={this.state}
                key={'dipListOfCi1'}
                label={'List Of CI'}
                modelName={'dipListOfCi1'}
                updateState={updateState}>
                <XlReactDipListOfCi
                    emptyPlaceholder={this.state.dipListOfCi1.emptyPlaceholder}
                    handlers={getHandlers()}
                    newItemSuffix={this.state.dipListOfCi1.newItemSuffix}
                    objLabel={'id'}
                    openOnClick={false}
                />
            </KitchenTestComponent>,
            dipMapStringString1: <KitchenTestComponent
                className="xl-react-dip-map-string-string-initial"
                currentState={this.state}
                key={'dipMapStringString1'}
                label={'Map string of strings'}
                modelName={'dipMapStringString1'}
                updateState={updateState}>
                <XlReactDipMapStringString/>
            </KitchenTestComponent>,
            dipSetOfCi1: <KitchenTestComponent
                className={'dip-set-of-ci-component'}
                currentState={this.state}
                key={'dipSetOfCi1'}
                label={'Set Of CI'}
                modelName={'dipSetOfCi1'}
                updateState={updateState}>
                <XlReactDipSetOfCi
                    emptyPlaceholder={this.state.dipSetOfCi1.emptyPlaceholder}
                    handlers={getHandlers()}
                    model={dataSetOfCi.data}
                    newItemSuffix={this.state.dipSetOfCi1.newItemSuffix}
                    objLabel={'id'}
                    openOnClick={false}
                />
            </KitchenTestComponent>,
            dipString1: <KitchenTestComponent
                className="xl-react-dip-string"
                currentState={this.state}
                key={'dipString1'}
                label={'Without default value'}
                modelName={'dipString1'}
                updateState={updateState}>
                <XlReactDipString/>
            </KitchenTestComponent>,
            dipString2: <KitchenTestComponent
                className="xl-react-dip-string-with-default-value"
                currentState={this.state}
                key={'dipString2'}
                label={'With default value'}
                modelName={'dipString2'}
                updateState={updateState}>
                <XlReactDipString/>
            </KitchenTestComponent>,
            dipStringSet1: <KitchenTestComponent
                className={'dip-set-of-strings-component'}
                currentState={this.state}
                key={'dipStringSet1'}
                label={'Set of strings'}
                modelName={'dipStringSet1'}
                updateState={updateState}>
                <XlReactDipSetOfStrings/>
            </KitchenTestComponent>
        };

        const isSingleItem = match && dips[match.params.name];

        return (
            <div className={classNames('dip-page', {'single-item': isSingleItem})}>
                <ComponentNavigator componentType="dip" state={this.state}/>
                {isSingleItem ? dips[match.params.name] : R.values(dips)}
            </div>
        );
    }
}
