import R from 'ramda';
import PropTypes from 'prop-types';
import React, {PureComponent} from 'react';
import {XlReactWidgetAutocomplete} from '../../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len
import history from '../../../history';

import './component-navigator.less';

export const componentTypes = {
    dip: 'dip',
    form: 'form',
    widget: 'widget'
};

export class ComponentNavigator extends PureComponent {
    static propTypes = {
        componentType: PropTypes.oneOf(R.values(componentTypes)).isRequired,
        state: PropTypes.object.isRequired
    };

    handleSuggestionSelected = ({key}) => {
        const {componentType} = this.props;
        history.push(`/${componentType}/${key}`);
    };

    autocompleteHandler = ({value}) => {
        const {state} = this.props;

        return Promise.resolve(
            R.pipe(
                R.keys,
                R.map((key) => ({key, value: state[key].navigationTitle})),
                R.filter((item) => R.includes(R.toLower(value), R.toLower(item.value)))
            )(state)
        );
    };

    autocompleteHandlers = {addCandidates: this.autocompleteHandler};

    render() {
        return <XlReactWidgetAutocomplete
            handlers={this.autocompleteHandlers}
            objLabel="value"
            onSuggestionSelected={this.handleSuggestionSelected}
        />;
    }
}
