import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import uuid from 'uuid-v4';
import classNames from 'classnames';

import {localeMessages} from '../../services/locale-service';
import {getModelValue} from '../../utils/components-utils';
import RU from '../../utils/ramda-utils';

export default class DisplayModeComponent extends PureComponent {
    static propTypes = {
        mode: PropTypes.string,
        onModeChange: PropTypes.func
    };

    constructor(props) {
        super(props);
        this.state = {
            groupName: uuid(),
            mode: getModelValue(this.props, 'mode', '')
        };
    }

    handleChange = (event) => {
        const val = event.target.value;
        this.setState({...this.state, mode: val});
        RU.invoke('onModeChange', [val], this.props);
    };

    render() {
        const {groupName, mode} = this.state;

        return <div className="displayMode">
            <span className="label">{localeMessages.displayModeLabel}</span>
            <span className="radioButtonGroup">
                <input
                    checked={mode === ''}
                    className={classNames('creation', {active: mode === ''})}
                    name={groupName}
                    onChange={this.handleChange}
                    type="radio"
                    value=""
                />Creation

                <input
                    checked={mode === 'edit'}
                    className={classNames('edit', {active: mode === 'edit'})}
                    name={groupName}
                    onChange={this.handleChange}
                    type="radio"
                    value="edit"
                />Edit

                <input
                    checked={mode === 'visual'}
                    className={classNames('display', {active: mode === 'visual'})}
                    name={groupName}
                    onChange={this.handleChange}
                    type="radio"
                    value="visual"
                />Display
            </span>
        </div>;
    }
}
