import R from 'ramda';
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {widgetStringValues} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string-enum';
import TestComponent from './test-component';
import ElementNavTitle from './element-nav-title';

export default class KitchenTestComponent extends Component {
    static propTypes = {
        children: PropTypes.element,
        className: PropTypes.string,
        currentState: PropTypes.object.isRequired,
        disabledToggles: PropTypes.array,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        modelName: PropTypes.string.isRequired,
        options: PropTypes.shape({copyModel: PropTypes.bool}),
        pristine: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.object
        ]),
        register: PropTypes.func,
        showBindings: PropTypes.bool,
        updateState: PropTypes.func.isRequired,
        valid: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.object
        ])
    };

    static defaultProps = {disabledToggles: [], options: {copyModel: true}, showBindings: true};

    constructor(props) {
        super(props);
        this.state = {changedParams: {}};
    }

    isRequired = (model) => R.pathOr(false, ['metadata', 'required'], model);

    render() {
        const {
            className, currentState, children, disabledToggles,
            label, modelName, options, showBindings, updateState
        } = this.props;
        const model = currentState[modelName];

        const {displayMode, extraBindingSections} = model;
        const {navigationDescription, navigationTitle} = model;
        const pristine = R.pathOr(true, ['pristine'], model);
        const valid = R.pathOr(true, ['valid'], model);
        const api = R.pathOr({}, ['api'], model);
        const disabled = R.pathOr(false, ['disabled'], model);
        const required = this.isRequired(model);
        const message = R.pathOr({show: false}, ['message'], model);
        const showMessages = R.pathOr(message.show, ['showMessages'], model);
        message.show = showMessages;
        const size = R.pathOr(widgetStringValues.SMALL, ['options', 'size'], model);

        const onDisplayModeChange = (val) => {
            currentState[modelName] = updateState(modelName, {...currentState[modelName], displayMode: val});
        };

        const onToggleFunction = (name, func) => !R.includes(name, disabledToggles) ? func : undefined;

        const onToggleDisabled = onToggleFunction('disabled', () => {
            currentState[modelName] = updateState(modelName, {
                ...currentState[modelName],
                disabled: !currentState[modelName].disabled
            });
        });

        const onToggleRequired = onToggleFunction('required', () => {
            const updatedPropName = 'metadata';
            updateState(modelName, {
                ...currentState[modelName],
                [updatedPropName]: {
                    ...currentState[modelName][updatedPropName],
                    required: !currentState[modelName][updatedPropName].required
                }
            });
        });

        const onToggleShowMessages = onToggleFunction('show-messages', () => {
            currentState[modelName] = updateState(modelName, {
                ...currentState[modelName],
                message: {
                    ...currentState[modelName].message,
                    show: !currentState[modelName].showMessages
                },
                showMessages: !currentState[modelName].showMessages
            });
        });

        const onModelChange = (val) => {
            currentState[modelName] = updateState(modelName, {...currentState[modelName], model: val});
        };

        const onSizeChange = (val) => {
            currentState[modelName] = updateState(modelName,
                R.assocPath(['metadata', 'size'], val, currentState[modelName])
            );
        };

        const onMessageChange = () => R.F;

        const onPristinityChange = (val) => {
            currentState[modelName] = updateState(modelName, {...currentState[modelName], pristine: val});
        };

        const onValidityChange = (val) => {
            currentState[modelName] = updateState(modelName, {...currentState[modelName], valid: val});
        };

        const onRegister = (val) => {
            currentState[modelName] = updateState(modelName, {...currentState[modelName], api: val});
        };

        const {copyModel} = options;

        const getModel = () => model.model && copyModel ? {model: model.model} : {};
        const getOnModelChange = () => copyModel ? {onModelChange} : {};

        const createNewParams = () => ({
            disabled: model.disabled,
            displayMode: model.displayMode,
            message: model.message,
            metadata: model.metadata,
            ...getModel(),
            onMessageChange,
            ...getOnModelChange(),
            onPristinityChange,
            onValidityChange,
            register: onRegister,
            showMessages: model.showMessages
        });

        const childrenWithProps = React.Children.map(children,
            (child) => React.cloneElement(child, R.mergeDeepRight(createNewParams(), this.state.changedParams))
        );

        const updateProps = (changedParams) => {
            this.setState({changedParams: R.mergeDeepRight(this.state.changedParams, changedParams)});
        };

        return <div>
            <ElementNavTitle
                description={navigationDescription}
                modelName={modelName}
                title={navigationTitle}
                type={model.type}
            />
            <h3>{label}</h3>
            <TestComponent
                api={api}
                className={className}
                disabled={disabled}
                displayMode={displayMode}
                extraBindingSections={extraBindingSections}
                hiddenDisplayedComponents={model.$hiddenDisplayedComponents || []}
                message={message}
                model={model.model}
                onModeChange={onDisplayModeChange}
                onModelChange={onModelChange}
                onSizeChange={onSizeChange}
                pristine={pristine}
                required={required}
                showBindings={showBindings}
                showMessages={showMessages}
                size={size}
                toggleDisable={onToggleDisabled}
                toggleRequired={onToggleRequired}
                toggleShowMessages={onToggleShowMessages}
                updateProps={updateProps}
                valid={valid}>
                {childrenWithProps}
            </TestComponent>
        </div>;
    }
}
