import R from 'ramda';
import {widgetReduxTableData} from '../widget-page/widget-initial-state';

export const fetchDataApi = (filter, paging, sorting) => {
    const mappedFilter = {...R.omit(['firstname-filter-key'], filter), firstname: filter['firstname-filter-key']};
    const mappedSorting = {...sorting, name: sorting.name === 'address-sort-key' ? 'address' : sorting.name};

    const {page, resultsPerPage} = paging;
    const {order, name} = mappedSorting;

    const sortFunc = R.sortWith([order === 'asc' ? R.ascend(R.prop(name)) : R.descend(R.prop(name))]);
    const sliceFunc = R.slice((page - 1) * resultsPerPage, page * resultsPerPage);

    const containsValue = (filterKey) => (value) => R.includes(R.toLower(mappedFilter[filterKey]))(R.toLower(value));

    const filterFunc = () => R.reduce(
        (acc, filterKey) =>
            mappedFilter[filterKey] ?
                R.filter(R.where({[filterKey]: containsValue(filterKey)}))(acc) : acc,
        widgetReduxTableData,
        R.keys(mappedFilter));

    const data = R.pipe(
        filterFunc,
        sortFunc
    )(widgetReduxTableData);

    return new Promise((resolve) => setTimeout(() => resolve({
        data: sliceFunc(data),
        headers: {'x-total-count': R.length(data)}
    }), 500));
};
