// eslint-disable-line max-lines
import classNames from 'classnames';
import {Promise} from 'es6-promise';
import PropTypes from 'prop-types';
import R from 'ramda';
import RS from 'ramdasauce';
import React, {PureComponent} from 'react';
import {connect} from 'react-redux';
import userInputImg from '../../images/user-input-grey.svg';
import {getBaseName} from '../../utils/ci-utils';
import {transformItemOr} from '../../utils/transform-utils';
import {XlReactWidgetAutocomplete} from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len
import {XlReactWidgetConfirmationDialog} from '../../xl-react-widget/xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog'; // eslint-disable-line max-len
import {XlReactWidgetDate} from '../../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import {XlReactWidgetFileUpload} from '../../xl-react-widget/xl-react-widget-file-upload/xl-react-widget-file-upload';
import {XlReactWidgetFilter} from '../../xl-react-widget/xl-react-widget-filter/xl-react-widget-filter';
import {XlReactWidgetInputAddon} from '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon';
import {XlReactWidgetInputAddonGroup} from '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon-group'; // eslint-disable-line max-len
import {XlReactWidgetModalDialog} from '../../xl-react-widget/xl-react-widget-modal-dialog/xl-react-widget-modal-dialog'; // eslint-disable-line max-len
import {XlReactWidgetProgressbar} from '../../xl-react-widget/xl-react-widget-progressbar/xl-react-widget-progressbar';
import {XlReactWidgetRadioGroup} from '../../xl-react-widget/xl-react-widget-radio-group/xl-react-widget-radio-group';
import {XlReactWidgetString} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactWidgetSwitch} from '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch';
import {XlReactWidgetSwitchButton} from '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch-button/xl-react-widget-switch-button'; // eslint-disable-line max-len
import {XlReactWidgetTags} from '../../xl-react-widget/xl-react-widget-tags/xl-react-widget-tags';
import {XlReactWidgetTime} from '../../xl-react-widget/xl-react-widget-time/xl-react-widget-time';
import {xlReactTreeRowRenderer} from '../../xl-react-widget/xl-react-widget-tree/row/xl-react-tree-row-renderer';
import {XlReactWidgetTree} from '../../xl-react-widget/xl-react-widget-tree/xl-react-widget-tree';
import kitchenSinkActions from '../actions/kitchen-sink-actions';
import widgetActions from '../actions/widget-actions';
import {getHandlers} from '../handlers/predefined-tags-autocomplete-handlers';
import AlertMessageTestComponent from '../playground/alert-message-test-component';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import KitchenTestComponent from '../playground/kitchen-test-component';
import {ReduxTableTestComponent} from '../playground/redux-table-test-component';
import ShowTestComponent from '../playground/show-test-component';
import {SpinnerTestComponent} from '../playground/spinner-test-component';
import {ORIENTATION, SplitPaneTestComponent} from '../playground/split-pane-test-component';
import {TableTestComponent} from '../playground/table-test-component';
import TreeSelectTestComponent from '../playground/tree-select-test-component';
import TreeTestComponent from '../playground/tree-test-component';
import WidgetCategoryTestComponent from '../playground/widget-category-test-component';
import {WidgetStringWithButtonAddonTestComponent} from '../playground/widget-string-with-button-addon-test-component';
import {autocompleteElements, widgetInitialState} from './widget-initial-state';

import './widget-page.less';

const tableId = 'redux-table-1';

const mapStateToProps = (state) => ({
    reduxTableInitialized: !RS.isNilOrEmpty(state.tables[tableId]),
    widgetContext: state.widget
});

const mapDispatchToProps = {
    initializeReduxTable: kitchenSinkActions.initializeReduxTable,
    updateModelState: widgetActions.updateModelState
};

@connect(mapStateToProps, mapDispatchToProps)
export class Widget extends PureComponent {
    static propTypes = {
        actions: PropTypes.object,
        initializeReduxTable: PropTypes.func,
        match: PropTypes.object,
        reduxTableInitialized: PropTypes.bool,
        updateModelState: PropTypes.func,
        widgetAlert1Type: PropTypes.string,
        widgetContext: PropTypes.object
    };

    state = widgetInitialState;

    updateState = (modelName, newState) => {
        this.props.updateModelState(modelName, newState);
    };

    render() {
        const {match} = this.props;

        const filterModel = '';

        const autocompleteHandler = ({value}) => {
            const term = transformItemOr('', value, 'id');
            const filterCond = (item) => R.toLower(item.id).indexOf(R.toLower(term)) !== -1;
            const res = R.pipe(
                R.filter(filterCond),
                R.sortBy(
                    R.compose(
                        R.toLower,
                        R.prop('id')
                    )
                )
            )(autocompleteElements);
            return Promise.resolve(res);
        };

        const delayedAutocompleteHandler = (valueModel) =>
            autocompleteHandler(valueModel).then((suggestions) => new Promise(
                (resolve) => setTimeout(() => resolve(suggestions), 10000)
            ));

        const autocompleteHandlers = {addCandidates: autocompleteHandler};
        const delayedAutocompleteHandlers = {addCandidates: delayedAutocompleteHandler};

        const widgets = {
            widgetAlert1: <AlertMessageTestComponent
                className={'widgetAlert1'}
                currentState={this.props.widgetContext}
                key={'widgetAlert1'}
                modelName="widgetAlert1"
                updateState={this.updateState}
            />,
            widgetAutocomplete1: <KitchenTestComponent
                className={'xl-react-widget-autocomplete-no-default-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete1'}
                label={'Autocomplete (string)'}
                modelName={'widgetAutocomplete1'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    metadata={this.props.widgetContext.widgetAutocomplete1.metadata}
                />
            </KitchenTestComponent>,
            widgetAutocomplete2: <KitchenTestComponent
                className={'xl-react-widget-autocomplete-width-default-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete2'}
                label={'Autocomplete (string) - with default value'}
                modelName={'widgetAutocomplete2'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    metadata={this.props.widgetContext.widgetAutocomplete1.metadata}
                />
            </KitchenTestComponent>,
            widgetAutocomplete3: <KitchenTestComponent
                className={'xl-react-widget-autocomplete-width-default-object-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete3'}
                label={'Autocomplete (string) - with default object value'}
                modelName={'widgetAutocomplete3'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    metadata={this.props.widgetContext.widgetAutocomplete1.metadata}
                    objLabel={'id'}
                />
            </KitchenTestComponent>,
            widgetAutocomplete4: <KitchenTestComponent
                className={'xl-react-widget-autocomplete-with-addons'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete4'}
                label={'Autocomplete - with input addons'}
                modelName={'widgetAutocomplete4'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    addons={
                        <XlReactWidgetInputAddonGroup>
                            <XlReactWidgetInputAddon>
                                <img alt={''} src={userInputImg}/>
                            </XlReactWidgetInputAddon>
                            <XlReactWidgetInputAddon>
                                2nd addon
                            </XlReactWidgetInputAddon>
                        </XlReactWidgetInputAddonGroup>
                    }
                    handlers={autocompleteHandlers}
                    metadata={this.props.widgetContext.widgetAutocomplete4.metadata}
                    objLabel={'id'}
                />
            </KitchenTestComponent>,
            widgetAutocomplete5: <KitchenTestComponent
                className={'xl-react-widget-autocomplete-delayed'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete5'}
                label={'Autocomplete (string)'}
                modelName={'widgetAutocomplete5'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={delayedAutocompleteHandlers}
                    metadata={this.props.widgetContext.widgetAutocomplete5.metadata}
                />
            </KitchenTestComponent>,
            widgetBottomSplitPane: <SplitPaneTestComponent
                className={'xl-react-widget-bottom-split-pane'}
                currentState={this.props.widgetContext}
                key={'widgetBottomSplitPane'}
                modelName={'widgetBottomSplitPane'}
                orientation={ORIENTATION.HORIZONTAL}
                updateState={this.updateState}>
                <section>
                    Main pane part
                </section>
                <section>
                    Bottom pane part
                </section>
            </SplitPaneTestComponent>,
            widgetCategory: <WidgetCategoryTestComponent
                categoryName={'Panel'}
                className="xl-react-widget-category-default"
                currentState={this.props.widgetContext}
                key={'widgetCategory'}
                modelName="widgetCategory"
                updateState={this.updateState}>
                <span>Panel context</span>
            </WidgetCategoryTestComponent>,
            widgetConfirmationDialog1: <ShowTestComponent
                className={'xl-react-widget-confirmation-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetConfirmationDialog1'}
                modelName={'widgetConfirmationDialog1'}
                updateState={this.updateState}>
                <XlReactWidgetConfirmationDialog>
                    <p className="confirmation-dialog-text">
                        {this.props.widgetContext.widgetConfirmationDialog1.text}
                    </p>
                </XlReactWidgetConfirmationDialog>
            </ShowTestComponent>,
            widgetDate1: <KitchenTestComponent
                className={'xl-react-widget-date-default'}
                currentState={this.props.widgetContext}
                key={'widgetDate1'}
                label={'Datepicker'}
                modelName={'widgetDate1'}
                updateState={this.updateState}>
                <XlReactWidgetDate
                    maxDate={this.props.widgetContext.widgetDate1.maximumDate}
                    minDate={this.props.widgetContext.widgetDate1.minimumDate}
                    model={this.props.widgetContext.widgetDate1.model}
                />
            </KitchenTestComponent>,
            widgetFileUpload1: <KitchenTestComponent
                className={'xl-react-widget-file-upload'}
                currentState={this.props.widgetContext}
                disabledToggles={['auto-focus']}
                key={'widgetFileUpload1'}
                label={'Upload file widget'}
                modelName={'widgetFileUpload1'}
                updateState={this.updateState}>
                <XlReactWidgetFileUpload/>
            </KitchenTestComponent>,
            widgetFilter1: <KitchenTestComponent
                className={'xl-react-widget-filter'}
                currentState={this.props.widgetContext}
                key={'widgetFilter1'}
                label={'My filter'}
                modelName={'widgetFilter1'}
                showBindings={false}
                updateState={this.updateState}>
                <XlReactWidgetFilter model={filterModel}/>
            </KitchenTestComponent>,
            widgetModalDialog1: <ShowTestComponent
                className={'xl-react-widget-modal-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetModalDialog1'}
                modelName={'widgetModalDialog1'}
                updateState={this.updateState}>
                <XlReactWidgetModalDialog>
                    <p className="widget-dialog-text">{this.props.widgetContext.widgetModalDialog1.text}</p>
                </XlReactWidgetModalDialog>
            </ShowTestComponent>,
            widgetModalDialog2: <ShowTestComponent
                className={'xl-react-widget-modal-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetModalDialog2'}
                modelName={'widgetModalDialog2'}
                updateState={this.updateState}>
                <XlReactWidgetModalDialog>
                    <p className="widget-dialog-text">{this.props.widgetContext.widgetModalDialog2.text}</p>
                </XlReactWidgetModalDialog>
            </ShowTestComponent>,
            widgetProgress1: <KitchenTestComponent
                className={'xl-react-widget-progressbar-default'}
                currentState={this.props.widgetContext}
                disabledToggles={['auto-focus', 'disabled', 'required', 'show-errors']}
                key={'widgetProgress1'}
                label={'With default values'}
                modelName={'widgetProgress1'}
                updateState={this.updateState}>
                <XlReactWidgetProgressbar
                    currentStatus={this.props.widgetContext.widgetProgress1.currentStatus}
                    maxValue={this.props.widgetContext.widgetProgress1.maxValue}
                    minValue={this.props.widgetContext.widgetProgress1.minValue}
                />
            </KitchenTestComponent>,
            widgetRadioGroup1: <KitchenTestComponent
                className="xl-react-widget-radio-group-default"
                currentState={this.props.widgetContext}
                key={'widgetRadioGroup1'}
                modelName="widgetRadioGroup1"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetRadioGroup name="Group" separatorLabel="or"/>
            </KitchenTestComponent>,
            widgetReduxTable: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable'}
                modelName={'widgetReduxTable'}
                showBindings={true}
                tableId={tableId}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetSpinner: <SpinnerTestComponent
                className={'xl-react-widget-spinner-test'}
                currentState={this.props.widgetContext}
                key={'widgetSpinner'}
                modelName={'widgetSpinner'}
                showBindings={true}
                updateState={this.updateState}
            />,
            widgetString1: <KitchenTestComponent
                className={'xl-react-widget-string-default'}
                currentState={this.props.widgetContext}
                key={'widgetString1'}
                label={'Input String'}
                modelName={'widgetString1'}
                updateState={this.updateState}>
                <XlReactWidgetString/>
            </KitchenTestComponent>,
            widgetString2: <KitchenTestComponent
                className={'xl-react-widget-string-with-addons'}
                currentState={this.props.widgetContext}
                key={'widgetString2'}
                label={'Input String with addon'}
                modelName={'widgetString2'}
                updateState={this.updateState}>
                <XlReactWidgetString
                    addons={
                        <XlReactWidgetInputAddonGroup>
                            <XlReactWidgetInputAddon>
                                <img alt={''} src={userInputImg}/>
                            </XlReactWidgetInputAddon>
                            <XlReactWidgetInputAddon>
                                2nd addon
                            </XlReactWidgetInputAddon>
                        </XlReactWidgetInputAddonGroup>
                    }
                />
            </KitchenTestComponent>,
            widgetStringWithButtonAddon: <WidgetStringWithButtonAddonTestComponent
                className={'xl-react-widget-string-with-button-addon'}
                currentState={this.props.widgetContext}
                key={'widgetStringWithButtonAddon'}
                label={'Input String with addon'}
                modelName={'widgetStringWithButtonAddon'}
                updateState={this.updateState}
            />,
            widgetSwitch: <KitchenTestComponent
                currentState={this.props.widgetContext}
                key={'widgetSwitch'}
                label={'Switch button component'}
                modelName={'widgetSwitch'}
                updateState={this.updateState}>
                <XlReactWidgetSwitch
                    model={this.props.widgetContext.widgetSwitch.model}
                    name="switch"
                    renderSwitch={
                        ({className}) =>
                            <XlReactWidgetSwitchButton className={className}>
                                Toggle switch
                            </XlReactWidgetSwitchButton>
                    }
                />
            </KitchenTestComponent>,
            widgetTable: <TableTestComponent
                currentState={this.props.widgetContext}
                key={'widgetTable'}
                modelName={'widgetTable'}
                showBindings={true}
                updateState={this.updateState}
            />,
            widgetTags1: <KitchenTestComponent
                className="xl-react-widget-tags"
                currentState={this.props.widgetContext}
                key={'widgetTags1'}
                label={'Tags'}
                modelName={'widgetTags1'}
                updateState={this.updateState}>
                <XlReactWidgetTags/>
            </KitchenTestComponent>,
            widgetTags2: <KitchenTestComponent
                className="xl-react-widget-tags"
                currentState={this.props.widgetContext}
                key={'widgetTags2'}
                label={'Tags with custom default error'}
                modelName={'widgetTags2'}
                updateState={this.updateState}>
                <XlReactWidgetTags/>
            </KitchenTestComponent>,
            widgetTags3: <KitchenTestComponent
                className="xl-react-widget-tags-with-autocomplete"
                currentState={this.props.widgetContext}
                key={'widgetTags3'}
                label={'Tags with Autocomplete input'}
                modelName={'widgetTags3'}
                updateState={this.updateState}>
                <XlReactWidgetTags
                    className="custom-tags-class"
                    handlers={{widgetTags3: getHandlers(this.props.widgetContext.widgetTags3.model)}}
                    inputType="autocomplete"
                    metadata={this.props.widgetContext.widgetTags3.metadata}
                    objLabel="value"
                    tagTransform={getBaseName}
                />
            </KitchenTestComponent>,
            widgetTime1: <KitchenTestComponent
                className={'xl-react-widget-time'}
                currentState={this.props.widgetContext}
                key={'widgetTime1'}
                label={'Timepicker'}
                modelName={'widgetTime1'}
                updateState={this.updateState}>
                <XlReactWidgetTime model={this.props.widgetContext.widgetTime1.model}/>
            </KitchenTestComponent>,
            widgetTree1: <TreeTestComponent
                className={'xl-react-widget-tree'}
                currentState={this.props.widgetContext}
                key={'widgetTree1'}
                modelName="widgetTree1"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetTree rowRenderer={xlReactTreeRowRenderer}/>
            </TreeTestComponent>,
            widgetTree2: <TreeTestComponent
                className={'xl-react-widget-tree-lazy'}
                currentState={this.props.widgetContext}
                key={'widgetTree2'}
                modelName="widgetTree2"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetTree rowRenderer={xlReactTreeRowRenderer}/>
            </TreeTestComponent>,
            widgetTreeSelect: <TreeSelectTestComponent
                currentState={this.props.widgetContext}
                key={'widgetTreeSelect'}
                modelName="widgetTreeSelect"
                showBindings={true}
            />,
            widgetVerticalSplitPane: <SplitPaneTestComponent
                className={'xl-react-widget-vertical-split-pane'}
                currentState={this.props.widgetContext}
                key={'widgetVerticalSplitPane'}
                modelName={'widgetVerticalSplitPane'}
                orientation={ORIENTATION.VERTICAL}
                updateState={this.updateState}>
                <section>
                    Left pane part
                </section>
                <section>
                    Main pane part
                </section>
            </SplitPaneTestComponent>
        };

        const isSingleItem = match && widgets[match.params.name];

        return <div className={classNames('widget-page', {'single-item': isSingleItem})}>
            <ComponentNavigator componentType="widget" state={this.props.widgetContext}/>
            {isSingleItem ? widgets[match.params.name] : R.values(widgets)}
        </div>;
    }
}

// eslint-enable-line max-lines
