import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {Button, ButtonGroup} from 'react-bootstrap';
import classNames from 'classnames';
import {getValue, getLabel} from './xl-react-toggle-button-service';

export class XlReactToggleButton extends PureComponent {
    static propTypes = {
        className: PropTypes.string,
        metadata: PropTypes.arrayOf(PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ])).isRequired,
        model: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ]),
        objLabel: PropTypes.string,
        objValue: PropTypes.string,
        onToggle: PropTypes.func.isRequired,
        type: PropTypes.string
    };

    static defaultProps = {type: 'secondary'};

    render() {
        const {
            className, metadata, model, type, onToggle, objValue, objLabel
        } = this.props;

        const btnGroupClassName = classNames('xl-react-button', `xl-${type}`, 'xl-react-toggle-button-group');

        const buttons = R.map(
            (buttonMetadata) => {
                const label = getLabel(buttonMetadata, objLabel);
                const buttonValue = getValue(buttonMetadata, objValue);

                const handleOnClick = () => {
                    onToggle(buttonValue);
                };
                return <Button
                    className={btnGroupClassName}
                    disabled={R.equals(model, buttonValue)}
                    key={label}
                    onClick={handleOnClick}>
                    {label}
                </Button>;
            }
        )(metadata);

        return <ButtonGroup className={className}>
            {buttons}
        </ButtonGroup>;
    }
}

export default XlReactToggleButton;
