import React from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import Sortable from 'react-sortablejs';
import '../../images/grab-handle.svg';

const SortableList = ({disabled, canSort, indexedItems, onChange, remove}) => {
    const handleRemove = (index) => () => remove(index);

    const removeButton = (index) =>
        disabled ?
            null :
            <span className="xl-react-btn-close" onClick={handleRemove(index)}/>;

    const listItems = R.map((current) => {
        const [val, index] = current;
        return <li className="xl-list-row" data-id={val} key={index}>
            <span className="text-container" title={val}>{val}</span>
            <span className="ui-sortable-handler-icon"/>
            {removeButton(index)}
        </li>;
    }, indexedItems);

    const disabledList = <ul>
        {listItems}
    </ul>;

    const sortableList = <Sortable className={canSort && 'ui-sortable'} onChange={onChange} tag="ul">
        {listItems}
    </Sortable>;

    return canSort ? sortableList : disabledList;
};

SortableList.propTypes = {
    canSort: PropTypes.bool,
    disabled: PropTypes.bool,
    indexedItems: PropTypes.array,
    onChange: PropTypes.func,
    remove: PropTypes.func
};

export default SortableList;
