import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';

import {resetPristinity, getModelValue} from '../../utils/components-utils';
import RU from '../../utils/ramda-utils';
import {isReadOnly} from '../../utils/display-mode';

import {transformItem, transformModel} from '../../utils/transform-utils';
import {
    computeInitialState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {prepareViewModel, addItem, moveItem} from './xl-react-dip-collection-of-ci-service';
import {XlReactDipCollectionOfCiComponent} from './xl-react-dip-collection-of-ci-component';

export class XlReactDipCollectionOfCi extends Component {
    static propTypes = {
        emptyPlaceholder: PropTypes.string,
        filterVisible: PropTypes.bool,
        handlers: PropTypes.object,
        inputVisible: PropTypes.bool,
        model: PropTypes.array,
        newItemSuffix: PropTypes.string,
        objLabel: PropTypes.string,
        openOnClick: PropTypes.bool,
        sortable: PropTypes.bool,
        unique: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        filterVisible: true,
        inputVisible: true,
        metadata: {},
        objLabel: 'id',
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {searchTerm: ''};
    }

    componentWillMount() {
        const model = getModelValue(this.props, 'model', []);
        const transformedModel = transformModel(this.props.objLabel)(model);
        computeInitialState(this, this.props, getValidators(this.props), transformedModel);
    }

    componentWillReceiveProps(nextProps) {
        const {pristine} = this.state;
        const model = !R.isNil(nextProps.model) ? nextProps.model : this.state.model;
        if (!R.isNil(model)) {
            const transformedModel = transformModel(this.props.objLabel)(model);
            computeNextState(this, nextProps, getValidators(nextProps), transformedModel, {pristine});
        }
    }

    onAddItem = (item) => {
        const model = addItem(this.state.model, transformItem(item, this.props.objLabel), this.props.unique);
        this.applyNewModel(model, [() => this.autoCompleteApi.erase()]);
    };

    onFilterUpdate = (searchTerm) => {
        this.setState({searchTerm});
    };

    onModelChange = (items, sortable, event) => {
        this.applyNewModel(moveItem(event.oldIndex, event.newIndex, this.state.model));
    };

    onRemoveItem = (itemIndex) => {
        this.applyNewModel(R.remove(itemIndex, 1, this.state.model));
    };

    api = {
        focus: () => this.autoCompleteApi.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config, () => {
            RU.invoke(['autoCompleteApi', 'resetPristinity'], [], this);
        })
    };

    applyNewModel = (model, customCallbacks = []) => {
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    };

    registerAutoCompleteApi = (autoCompleteApi) => {
        this.autoCompleteApi = autoCompleteApi;
    };

    render() {
        const {
            disabled, displayMode, filterVisible, handlers, inputVisible, metadata,
            objLabel, onMessageChange, onValidityChange, sortable
        } = this.props;
        const {message, model, searchTerm, valid} = this.state;

        const filteredItems = prepareViewModel(model, searchTerm, objLabel);
        const readOnly = isReadOnly(displayMode, disabled);

        return <XlReactDipCollectionOfCiComponent {...({
            filteredItems,
            filterVisible,
            handleAddItem: this.onAddItem,
            handleFilterUpdate: this.onFilterUpdate,
            handleModelChange: this.onModelChange,
            handleRemoveItem: this.onRemoveItem,
            handlers,
            inputVisible,
            message,
            metadata,
            model,
            objLabel,
            onMessageChange,
            onValidityChange,
            readOnly,
            registerAutoCompleteApi: this.registerAutoCompleteApi,
            searchTerm,
            sortable,
            valid
        })}
        />;
    }
}
