import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import moment from 'moment';
import classNames from 'classnames';

import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactWidgetDate} from '../../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import {XlReactWidgetTime} from '../../xl-react-widget/xl-react-widget-time/xl-react-widget-time';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';

import './xl-react-dip-datetime.less';

export class XlReactDipDatetimeComponent extends PureComponent {
    static propTypes = {
        date: PropTypes.object,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        handleDateChange: PropTypes.func,
        handleTimeChange: PropTypes.func,
        invalid: PropTypes.bool,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        messageHandler: PropTypes.func,
        model: PropTypes.object,
        onMessageChange: PropTypes.func,
        onPristinityChange: PropTypes.func,
        onValidityChange: PropTypes.func,
        register: PropTypes.func,
        required: PropTypes.bool,
        time: PropTypes.object
    };

    handleDateChange = (datetime) => {
        if (!R.equals(moment(this.props.model), moment(datetime))) {
            this.props.handleDateChange(datetime);
        }
    };

    render() {
        const {
            date, description, disabled, displayMode, invalid, messageHandler, message,
            handleTimeChange, label, model, onPristinityChange,
            onValidityChange, register, required, time
        } = this.props;

        const getModel = () => moment(model).isValid() ? model : null;

        const controlClass = classNames('xl-react-dip-datetime', {invalid});

        return <XlReactFormItemControl {...({
            controlClass,
            description,
            label,
            message,
            required
        })}>
            <XlReactWidgetDate {...({
                disabled,
                displayMode,
                messageHandler,
                metadata: {date, required},
                model,
                onModelChange: this.handleDateChange,
                onPristinityChange: R.curry(onPristinityChange)('date'),
                onValidityChange: R.curry(onValidityChange)('date'),
                register: R.curry(register)('date'),
                showParentMessages: true
            })}
            />
            <XlReactWidgetTime {...({
                disabled,
                displayMode,
                metadata: time,
                model: getModel(),
                onModelChange: handleTimeChange,
                onPristinityChange: R.curry(onPristinityChange)('time'),
                onValidityChange: R.curry(onValidityChange)('time'),
                register: R.curry(register)('time'),
                showParentMessages: true
            })}
            />
        </XlReactFormItemControl>;
    }
}
