import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import {Promise} from 'es6-promise';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactWidgetAutocomplete} from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {isValidEnumValue} from '../../services/validate-service';
import {validationMessageClasses} from '../../utils/css-utils';
import {hasInputHints, getInputHintValues} from '../../utils/ci-utils';

import './xl-react-dip-enum.less';

export class XlReactDipEnumComponent extends PureComponent {
    static propTypes = {
        disabled: PropTypes.bool,
        message: xlMessageTypeProp,
        messageHandler: PropTypes.func,
        metadata: PropTypes.object,
        model: PropTypes.string,
        onMessageChange: PropTypes.func,
        onModelChange: PropTypes.func,
        onPristinityChange: PropTypes.func,
        onValidityChange: PropTypes.func,
        registerChildApi: PropTypes.func,
        valid: PropTypes.bool
    };

    handleModelChange = (model, initial) => {
        const isSelectedFromSuggestions = this.isInputHintsPresent ?
            R.findIndex(R.propEq('label', model))(this.getEnumValues()) > -1 :
            R.includes(model, this.getEnumValues());

        if (isSelectedFromSuggestions) {
            this.onSuggestionSelection(model, initial);
        } else {
            this.props.onModelChange(model, initial);
        }
    };

    onSuggestionSelection = (model, initial) => {
        const componentModel = this.isInputHintsPresent ?
            R.pipe(
                R.find(R.propEq('label', model)),
                R.prop('value')
            )(this.getEnumValues()) :
            model;
        this.props.onModelChange(componentModel, initial);
    };

    getEnumValues = () => this.isInputHintsPresent ?
        getInputHintValues(this.props.metadata) : this.props.metadata.enumValues;

    render() {
        const {
            disabled, messageHandler, message, metadata, model, onMessageChange,
            onPristinityChange, onValidityChange, registerChildApi, valid
        } = this.props;

        this.isInputHintsPresent = hasInputHints(metadata);

        const enumValues = this.getEnumValues();

        const handlers = {
            addCandidates: ({value}) => {
                if (this.isInputHintsPresent) {
                    const hasSearchedTerm = (enumVal) =>
                        R.includes(R.toLower(value), R.toLower(R.prop('label', enumVal)));
                    const filteredItems = R.filter(hasSearchedTerm, enumValues);
                    return new Promise((resolve) => {
                        resolve(filteredItems.sort());
                    });
                }

                const hasSearchedTerm = (enumVal) => R.includes(R.toLower(value), R.toLower(enumVal));
                return new Promise((resolve) => {
                    const filteredItems = R.filter(hasSearchedTerm, enumValues);
                    resolve(R.map((name) => ({id: name}))(filteredItems.sort()));
                });
            }
        };

        const invalid = !isValidEnumValue({checkedValue: model, enumValues});

        const controlClass = classNames(
            {'xl-react-dip-enum': true},
            validationMessageClasses(valid, message)
        );

        return <XlReactFormItemControl {...({...metadata, controlClass, message})}>
            <XlReactWidgetAutocomplete {...({
                disabled,
                handlers,
                invalid,
                message,
                messageHandler,
                metadata: R.omit(['label', 'description'], metadata),
                model,
                objLabel: this.isInputHintsPresent ? 'label' : 'id',
                onMessageChange,
                onModelChange: this.handleModelChange,
                onPristinityChange,
                onValidityChange,
                register: registerChildApi
            })}
            />
        </XlReactFormItemControl>;
    }
}
