import PropTypes from 'prop-types';
import React, {Component} from 'react';
import {
    getModelValue,
    isComponentDisabled,
    resetPristinity
} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipIntegerComponent} from './xl-react-dip-integer-component';

export class XlReactDipInteger extends Component {
    static propTypes = {
        model: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
        ...XlReactDipContainerProps
    };
    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {};
    }

    componentWillMount() {
        const model = getModelValue(this.props, 'model', '');
        computeInitialState(this, this.props, getValidators(this.props), model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(
            this,
            nextProps,
            this.state,
            getValidators(nextProps)
        );
    }

    onChange = (event) => {
        this.processChange(event.target.value);
    };

    onErase = () => {
        this.processChange('');
    };

    api = {
        focus: () => this.inputField.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    assignRef = (input) => {
        this.inputField = input;
    };

    processChange = (model) => {
        if ((/^\d+$|^$/).test(model)) {
            computeNextState(
                this,
                this.props,
                getValidators(this.props),
                model
            );
        }
    };

    render() {
        const {
            metadata: {description, label, placeholder, required},
            onMessageChange, onValidityChange
        } = this.props;
        const {message, model, valid} = this.state;

        return (
            <XlReactDipIntegerComponent
                {...{
                    assignRef: this.assignRef,
                    description,
                    disabled: isComponentDisabled(this.props),
                    handleChange: this.onChange,
                    handleErase: this.onErase,
                    label,
                    message,
                    model,
                    onMessageChange,
                    onValidityChange,
                    placeholder,
                    required,
                    valid
                }}
            />
        );
    }
}
