import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {getModelValue, isDisabled, resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {transformItem} from '../../utils/transform-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipListComponent} from './xl-react-dip-list-component';

export class XlReactDipList extends Component {
    static propTypes = {
        handlers: PropTypes.object,
        model: PropTypes.array,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {};
    }

    componentWillMount() {
        const model = getModelValue(this.props, 'model', []);
        computeInitialState(this, this.props, getValidators(this.props), model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
    }

    handleRemoveSortableListItem = (itemKey) => {
        this.notify(R.remove(itemKey, 1, this.state.model));
    };

    onAddItem = () => {
        const {typed} = this.state;
        this.setState({typed: ''});
        this.notify([...this.state.model, typed]);
    };

    onSortableListChange = (model) => {
        this.notify(model);
    };

    onSuggestionSelection = (item) => {
        this.notify([...this.state.model, transformItem(item, this.props.objLabel)], [() => this.childApi.erase()]);
    };

    onUserInput = (inputText) => {
        this.setState({typed: inputText});
    };

    api = {
        focus: () => this.childApi.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config, () => {
            this.childApi.resetPristinity(config);
        })
    };

    notify(model, customCallbacks = []) {
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    }

    registerChildApi = (childApi) => {
        this.childApi = childApi;
    };

    render() {
        const {
            displayMode,
            handlers,
            metadata,
            objLabel,
            onMessageChange,
            onValidityChange
        } = this.props;
        const {description, label, name, required, validators} = metadata;
        const {message, model, typed, valid} = this.state;

        const autocompleteMetadata = R.omit(['default', 'description', 'label', 'required'], metadata);
        const disabled = isDisabled(this.props.disabled, this.props.displayMode);

        return <XlReactDipListComponent {...({
            autocompleteMetadata,
            description,
            disabled,
            displayMode,
            handleAddItem: this.onAddItem,
            handlers: handlers && handlers[name],
            handleSortableListChange: this.onSortableListChange,
            handleSuggestionSelection: this.onSuggestionSelection,
            handleUserInput: this.onUserInput,
            label,
            message,
            model,
            name,
            objLabel,
            onMessageChange,
            onRemoveSortableListItem: this.handleRemoveSortableListItem,
            onValidityChange,
            registerChildApi: this.registerChildApi,
            required,
            typed,
            valid,
            validators
        })}
        />;
    }
}
