import R from 'ramda';
import React from 'react';
import PropTypes from 'prop-types';
import {Column} from 'react-virtualized';
import {localeMessages} from '../../../services/locale-service';
import {InputValue} from '../input-value';
import {MapValueInput} from '../map-value-input';
import {COLUMNS, shouldFocusInput} from '../map-table-utils';
import {headerRenderer} from './header-renderer';

export const tableValueColumn = (props) => {
    const {
        columnWidth,
        cellState,
        disabled,
        handleCloseButton,
        handleNewValueBlur,
        handleNewValueFocus,
        handleNewValueKeyDown,
        handleRemoveKey,
        handleValueChange,
        inputType,
        newValue,
        showNewKeyValueRow,
        showRemoveButton,
        updateCellState
    } = props;

    const cellRenderer = (cellProps) => {
        const {rowIndex, cellData, rowData} = cellProps;
        const focusInput = shouldFocusInput(rowIndex, COLUMNS.value, cellState);

        const handleUpdateCellState = (value) => {
            updateCellState({column: COLUMNS.value, row: rowIndex, value});
        };

        const onFocusNewValue = (event) => {
            handleUpdateCellState(event.target.value);
            handleNewValueFocus(event);
        };

        const shouldDisplayInputValue = () => R.and(R.equals(rowIndex, 0), showNewKeyValueRow);

        if (shouldDisplayInputValue()) {
            return <InputValue
                disabled={disabled}
                handleCloseButton={handleCloseButton}
                handleNewValueBlur={handleNewValueBlur}
                handleNewValueFocus={onFocusNewValue}
                handleNewValueKeyDown={handleNewValueKeyDown}
                inputType={inputType}
                model={newValue}
                shouldFocus={focusInput}
                showRemoveButton={showRemoveButton}
            />;
        }

        const getChangedValue = () => focusInput ? cellState.value : undefined;

        return !R.isNil(cellData) && <MapValueInput
            changedValue={getChangedValue()}
            disabled={disabled}
            handleRemoveKey={handleRemoveKey}
            handleValueChange={handleValueChange}
            inputType={inputType}
            itemKey={rowData.key}
            shouldFocus={focusInput}
            updateCellState={handleUpdateCellState}
            value={cellData}
        />;
    };

    return <Column
        cellRenderer={cellRenderer}
        className="xl-virtualized-table-cell"
        dataKey="value"
        flexGrow={1}
        headerRenderer={headerRenderer(localeMessages.valueLabel, COLUMNS.value)}
        width={columnWidth}
    />;
};

tableValueColumn.propTypes = {
    cellState: PropTypes.shape({
        column: PropTypes.oneOf([COLUMNS.key, COLUMNS.value]).isRequired,
        row: PropTypes.number.isRequired,
        value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
    }),
    columnWidth: PropTypes.number.isRequired,
    disabled: PropTypes.bool.isRequired,
    handleCloseButton: PropTypes.func.isRequired,
    handleNewValueBlur: PropTypes.func.isRequired,
    handleNewValueFocus: PropTypes.func.isRequired,
    handleNewValueKeyDown: PropTypes.func.isRequired,
    handleRemoveKey: PropTypes.func.isRequired,
    handleValueChange: PropTypes.func.isRequired,
    inputType: PropTypes.oneOf(['password', 'text']).isRequired,
    model: PropTypes.string,
    newValue: PropTypes.string.isRequired,
    showNewKeyValueRow: PropTypes.bool.isRequired,
    showRemoveButton: PropTypes.bool.isRequired,
    updateCellState: PropTypes.func.isRequired
};
