import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {localeMessages} from '../../services/locale-service';
import {blurFieldOnSubmit} from '../input/input-service';

export class InputValue extends PureComponent {
    static propTypes = {
        disabled: PropTypes.bool.isRequired,
        handleCloseButton: PropTypes.func.isRequired,
        handleNewValueBlur: PropTypes.func.isRequired,
        handleNewValueFocus: PropTypes.func.isRequired,
        handleNewValueKeyDown: PropTypes.func.isRequired,
        inputType: PropTypes.oneOf(['password', 'text']).isRequired,
        model: PropTypes.string,
        shouldFocus: PropTypes.bool.isRequired,
        showRemoveButton: PropTypes.bool.isRequired
    };

    componentDidMount() {
        if (this.props.shouldFocus && this.inputField) {
            this.inputField.focus({preventScroll: true});
        }
    }

    handleKeyDown = (event) => {
        blurFieldOnSubmit(this.inputField, event);
    };

    assignInputValueRef = (input) => {
        this.inputField = input;
    };

    inputValueInput = (
        {
            disabled,
            handleCloseButton,
            handleNewValueBlur,
            handleNewValueFocus,
            handleNewValueKeyDown,
            inputType,
            model,
            showRemoveButton
        }
    ) =>
        <div className="value-div new-value-div">
            {inputType === 'password' ?
                <input
                    disabled={disabled}
                    name="add-value"
                    onBlur={handleNewValueBlur}
                    onChange={handleNewValueKeyDown}
                    onFocus={handleNewValueFocus}
                    onKeyDown={this.handleKeyDown}
                    placeholder={localeMessages.valuePlaceholder}
                    ref={this.assignInputValueRef}
                    type={inputType}
                    value={model}
                /> :
                <textarea
                    disabled={disabled}
                    name="add-value"
                    onBlur={handleNewValueBlur}
                    onChange={handleNewValueKeyDown}
                    onFocus={handleNewValueFocus}
                    onKeyDown={this.handleKeyDown}
                    placeholder={localeMessages.valuePlaceholder}
                    ref={this.assignInputValueRef}
                    value={model}
                />
            }
            {
                showRemoveButton &&
                <div
                    className="xl-react-btn-close"
                    onClick={handleCloseButton}
                />
            }
        </div>;

    render() {
        const inputValueSpan = <span className="input-value">{localeMessages.valuePlaceholder}</span>;
        return this.props.disabled ? inputValueSpan : this.inputValueInput(this.props);
    }
}
