import R from 'ramda';
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {blurFieldOnSubmit} from '../input/input-service';
import {isTextType} from './xl-react-dip-map-string-string-constants';
import {updateState} from './map-table-utils';

export class MapValueInput extends Component {
    static propTypes = {
        changedValue: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        disabled: PropTypes.bool.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        handleValueChange: PropTypes.func.isRequired,
        inputType: PropTypes.string.isRequired,
        itemKey: PropTypes.string.isRequired,
        shouldFocus: PropTypes.bool.isRequired,
        updateCellState: PropTypes.func.isRequired,
        value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
    };

    constructor(props) {
        super(props);
        const {value, changedValue} = props;
        if (!R.isNil(changedValue) && changedValue !== value) {
            this.state = {internalValue: changedValue};
        } else {
            this.state = {internalValue: value};
        }
    }

    componentDidMount() {
        if (this.props.shouldFocus && this.inputField) {
            this.inputField.focus({preventScroll: true});
        }
    }

    componentWillReceiveProps(nextProps) {
        updateState(this, nextProps);
    }

    componentWillUnmount() {
        const {handleValueChange, itemKey, value} = this.props;
        const {internalValue} = this.state;
        if (!R.equals(value, internalValue)) {
            handleValueChange(itemKey, internalValue);
        }
    }

    handleKeyBlur = ({target: {value}}) => {
        const {handleValueChange, itemKey} = this.props;
        handleValueChange(itemKey, value);
    };

    handleChange = ({target: {value}}) => {
        this.setState({internalValue: value}, () => {
            this.props.updateCellState(this.state.internalValue);
        });
    };

    handleFocus = () => {
        this.props.updateCellState(this.state.internalValue);
    };

    handleRemoveKey = () => {
        const {handleRemoveKey, itemKey} = this.props;
        handleRemoveKey(itemKey);
    };

    handleKeyDown = (event) => {
        blurFieldOnSubmit(this.inputField, event);
    };

    assignInputValueRef = (input) => {
        this.inputField = input;
    };

    getTitle = (inputType, value) => isTextType(inputType) ? value : null;

    getInputComponent = ({disabled, inputType, value}, {internalValue}) => inputType === 'password' ?
        <input
            onBlur={this.handleKeyBlur}
            onChange={this.handleChange}
            onFocus={this.handleFocus}
            onKeyDown={this.handleKeyDown}
            readOnly={disabled}
            ref={this.assignInputValueRef}
            title={this.getTitle(inputType, value)}
            type={inputType}
            value={internalValue}
        /> :
        <textarea
            onBlur={this.handleKeyBlur}
            onChange={this.handleChange}
            onFocus={this.handleFocus}
            onKeyDown={this.handleKeyDown}
            readOnly={disabled}
            ref={this.assignInputValueRef}
            title={this.getTitle(inputType, value)}
            value={internalValue}
        />;

    render() {
        const {disabled} = this.props;

        const closeBtn = <div
            className="xl-react-btn-close"
            onClick={this.handleRemoveKey}
        />;
        return <div className="value-div">
            {this.getInputComponent(this.props, this.state)}
            {disabled ? null : closeBtn}
        </div>;
    }
}
