import R from 'ramda';
import RU from '../utils/ramda-utils';

export const findCategoryNameByPropertyName = (metadata, propName) => R.pipe(
    R.find((val) => val.name === propName),
    (val) => R.isNil(val) ? val : val.category
)(metadata.properties);

const isGroupOpen = (groupProperties, collapsed) => {
    if (collapsed === 'auto') {
        const {required, hiddenRequired} = RU.countHits({
            hiddenRequired: R.whereEq({hidden: true, required: true}),
            required: R.propEq('required', true)
        })(groupProperties);
        return required - hiddenRequired > 0;
    }
    return !R.isNil(collapsed) ? collapsed : true;
};

export const getFieldNamesWithShownErrors = (invalidFieldNames, dirtyFieldNames, message) => {
    if (RU.isTrue(['show'], message)) {
        return invalidFieldNames;
    }
    return R.intersection(invalidFieldNames, dirtyFieldNames);
};

export const getInvalidCategoryNames = ({groupedProperties = [], pristinity, message, validity}) => {
    const invalidFieldNames = R.keys(R.filter(R.not, validity));
    const dirtyFieldNames = R.keys(R.filter(R.not, pristinity));
    const fieldNamesWithShownErrors = getFieldNamesWithShownErrors(invalidFieldNames, dirtyFieldNames, message);
    return R.reduce((acc, cat) => {
        const categoryPropNames = R.map(R.prop('name'), cat.properties);
        if (!R.isEmpty(R.intersection(fieldNamesWithShownErrors, categoryPropNames))) {
            return [...acc, cat.category];
        }
        return acc;
    }, [], groupedProperties);
};

export const groupByCategory = (dynamicFormProperties, collapsed, oldGroupedProperties) => {
    const defaultToCommonGroup = R.map(R.over(R.lensProp('category'), R.defaultTo('Common')));
    const createGroupItem = R.map(([category, properties]) => ({
        category,
        isOpen: oldGroupedProperties ? R.prop('isOpen', R.find(R.propEq('category', category), oldGroupedProperties)) :
            isGroupOpen(properties, collapsed),
        properties
    }));
    const sortByGroupName = R.sort(
        R.comparator((left, right) => left.category === 'Common' || left.category < right.category)
    );

    return R.pipe(
        defaultToCommonGroup,
        R.groupBy(R.prop('category')),
        R.toPairs,
        createGroupItem,
        sortByGroupName
    )(dynamicFormProperties);
};

export const prepareGroupedProperties = (metadata, collapsed, previousGroupedProperties) => {
    const isNonHidden = (property) => R.isNil(property.hidden) || property.hidden === false;

    const makeRequiredFalseIfAsContainment = (property) => {
        property.required = property.asContainment ? false : property.required;
        return property;
    };

    const getProperties = (obj) => obj.properties;

    return groupByCategory(
        R.pipe(getProperties, R.filter(isNonHidden), R.map(makeRequiredFalseIfAsContainment))(metadata),
        collapsed,
        previousGroupedProperties
    );
};
