import {delay} from 'redux-saga';
import {INPUT_DEBOUNCE_TIME} from '../xl-react-widget-redux-table-constants';
import tableActions from '../xl-react-widget-redux-table-actions';
import {extractTotal} from '../../../utils/http-utils';
import {getTableById} from '../xl-react-widget-table-selectors';
import {buildFilterQuery, buildSortQuery} from '../query-builder-utils';
import {paginateSaga} from './xl-react-widget-redux-table-pagination-saga';
import {call, select, put} from 'redux-saga/effects';

export function* withDelay(effect, timeout = INPUT_DEBOUNCE_TIME) {
    yield call(delay, timeout);
    yield effect;
}

export function* fetchData(tableId, api) {
    const {filters, layout, paging, sorting} = yield select(getTableById, tableId);
    const filterQuery = buildFilterQuery(filters);
    const sortQuery = buildSortQuery(sorting, layout);
    try {
        const {data, headers} = yield call(api, filterQuery, paging, sortQuery);
        yield put(tableActions.show({
            data,
            tableId,
            total: extractTotal(headers)
        }));
    } catch (error) {
        yield put(tableActions.show({error, tableId}));
    }
}

export function* withSpinner(tableId, effect) {
    try {
        yield put(tableActions.changeLoadingState(tableId, true));
        yield effect;
        yield put(tableActions.changeLoadingState(tableId, false));
    } catch (error) {
        yield put(tableActions.changeLoadingState(tableId, false));
    }
}

export function* initializeSaga(action) {
    if (action.api) {
        yield call(withSpinner, action.tableId, call(fetchData, action.tableId, action.api));
    }
}

export function* refreshSaga({api, tableId}) {
    const {paging: {resultsPerPage}} = yield select(getTableById, tableId);
    yield call(paginateSaga, {api, page: 1, resultsPerPage, tableId});
}
