/* eslint-disable */
import React from 'react';
import PropTypes from 'prop-types';
import RU from '../../utils/ramda-utils';
import {buildColumns} from './filter-element-position-utils';
import {XlReactWidgetAutocomplete} from '../xl-react-widget-autocomplete/xl-react-widget-autocomplete';
import {XlReactWidgetString} from '../xl-react-widget-string/xl-react-widget-string';
import {XlReactWidgetTags} from '../xl-react-widget-tags/xl-react-widget-tags';
import {tableFiltersType} from '../xl-react-widget-table/xl-react-widget-table-types';

import './xl-react-widget-table-filters.less';

export default class XlReactWidgetTableFilters extends React.Component {
    static propTypes = {
        config: tableFiltersType,
        onFilterChange: PropTypes.func.isRequired
    };

    handleModelChange = (key) => (model) => {
        if (this.props.config.find((filter) => filter.key === key).model !== model) {
            this.props.onFilterChange({type: 'single', change: {key, model}});
        }
    };

    filters = {
        autocomplete: ({className, key, label, model, placeholder}) =>
            <XlReactWidgetAutocomplete
                className={className}
                handlers={getAllDeploymentTaskTypesHandlers}
                menuHeight={350}
                metadata={{label: label, placeholder: placeholder}}
                model={model}
                onModelChange={this.handleModelChange(key)}
            />,
        input: ({className, key, label, model, placeholder}) =>
            <XlReactWidgetString
                className={className}
                inputIconRenderer={() => <i className="search-icon glyphicon glyphicon-search"/>}
                metadata={{
                    label: label,
                    placeholder: placeholder
                }}
                model={model}
                onModelChange={this.handleModelChange(key)}
            />,
        tags: ({className, key, label, model, placeholder}) =>
            <XlReactWidgetTags
                className={className}
                handlers={{autocomplete: getHandlers}}
                inputType="autocomplete"
                metadata={{
                    label: label,
                    name: 'autocomplete',
                    placeholder: placeholder
                }}
                model={model}
                objLabel="status"
                onModelChange={this.handleModelChange(key)}
                skin="color-and-checkbox"
                tagTransform={R.identity}
            />

    };

    renderColumns = () => {
        const renderFilterPanelColumn = (column, index) =>
            <div className={`filters-panel-column ${index}`} key={`key-${index}`}>
                {column}
            </div>;

        return RU.mapIndexed(renderFilterPanelColumn, buildColumns(this.filters, this.props.config));
    };

    render() {
        return <div className="xl-react-redux-table-filters-panel">{this.renderColumns()}</div>;
    }
}
