import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {XlReactWidgetMultiLinePasswordComponent} from './xl-react-widget-multi-line-password-component';
import {XlReactWidgetInputComponent} from './xl-react-widget-input-component';
import {XlReactWidgetSizedStringComponent} from './xl-react-widget-sized-string-component';
import {widgetStringValues} from './xl-react-widget-string-enum';

import './xl-react-widget-string.less';

export class XlReactWidgetStringComponent extends PureComponent {
    static propTypes = {
        addons: PropTypes.oneOfType([
            PropTypes.node,
            PropTypes.arrayOf(PropTypes.node)
        ]),
        assignRef: PropTypes.func,
        autocomplete: PropTypes.string,
        canRevealPassword: PropTypes.bool,
        className: PropTypes.string,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        inputIconRenderer: PropTypes.func,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        model: PropTypes.string,
        name: PropTypes.string,
        onBlur: PropTypes.func,
        onErase: PropTypes.func,
        onKeyPress: PropTypes.func,
        onModelChange: PropTypes.func,
        onStageModel: PropTypes.func,
        onUnstageModel: PropTypes.func,
        placeholder: PropTypes.string,
        pristine: PropTypes.bool,
        required: PropTypes.bool,
        requiredDirection: PropTypes.string,
        size: PropTypes.string,
        toggleButton: PropTypes.node,
        type: PropTypes.string,
        valid: PropTypes.bool
    };

    static defaultProps = {
        disabled: false,
        model: '',
        required: false,
        type: 'text'
    };

    state = {
        containerFocused: false,
        focused: false
    };

    handleOnFocus = (event) => {
        this.setState({focused: true}, () => RU.invoke(['onFocus'], [event], this.props));
    };

    handleContainerFocus = () => {
        this.setState({containerFocused: true});
    };

    handleContainerBlur = () => {
        this.setState({containerFocused: false});
    };

    handleOnBlur = (event) => {
        this.setState({focused: false}, () => RU.invoke(['onBlur'], [event], this.props));
    };

    handleOnKeyPress = (event) => {
        RU.invoke(['onKeyPress'], [event], this.props);
    };

    render() {
        const {
            addons,
            assignRef,
            autocomplete,
            className,
            disabled,
            message,
            model,
            name,
            onErase,
            onModelChange,
            onStageModel,
            onUnstageModel,
            placeholder,
            required,
            size,
            toggleButton,
            type,
            valid
        } = this.props;

        const {focused} = this.state;

        const inputClasses = classNames(
            {'readonly-input': disabled},
            validationMessageClasses(valid, message)
        );

        const inputWrapperClasses = classNames('xl-react-component-input-wrapper', className);
        const createInputField = () =>
            <XlReactWidgetInputComponent
                assignRef={assignRef}
                autocomplete={autocomplete}
                className={inputWrapperClasses}
                disabled={disabled}
                focused={focused}
                inputClasses={inputClasses}
                model={model}
                name={name}
                onBlur={this.handleOnBlur}
                onFocus={this.handleOnFocus}
                onKeyPress={this.handleOnKeyPress}
                onModelChange={onModelChange}
                placeholder={placeholder}
                required={required}
                toggleButton={toggleButton}
                type={type}
            />;

        const pickComponentToRender = R.cond([
            [
                R.includes(R.__, [widgetStringValues.LARGE, widgetStringValues.MEDIUM]),
                () => {
                    if (R.equals(type, 'password')) {
                        return <XlReactWidgetMultiLinePasswordComponent
                            {...this.props}
                            className={inputWrapperClasses}
                            focused={focused}
                            inputClasses={inputClasses}
                            onBlur={this.handleOnBlur}
                            onFocus={this.handleOnFocus}
                            onStageModel={onStageModel}
                            onUnstageModel={onUnstageModel}
                        />;
                    }
                    return <XlReactWidgetSizedStringComponent
                        {...this.props}
                        className={inputWrapperClasses}
                        focused={focused}
                        inputClasses={inputClasses}
                        onBlur={this.handleOnBlur}
                        onFocus={this.handleOnFocus}
                    />;
                }
            ],
            [
                R.T,
                R.always(createInputField())
            ]
        ]);

        const closeFieldClassName = classNames(size, 'xl-react-btn-close');

        const closeField = () => !disabled && !R.isEmpty(model) ?
            <span className={closeFieldClassName} onClick={onErase}/> : null;

        const inputContainerClassName = classNames(
            'xl-react-widget-string-input-container',
            {
                'xl-react-widget-string-input-container--focus': this.state.containerFocused,
                'xl-react-widget-string-input-group': !R.isNil(addons)
            }
        );

        return <XlReactFormItemControl {...({
            ...this.props,
            controlClass: classNames(
                className,
                'xl-react-widget-string',
                {focused: focused}
            )
        })}>
            <div
                className={inputContainerClassName}
                onBlur={this.handleContainerBlur}
                onFocus={this.handleContainerFocus}>
                {addons}
                {pickComponentToRender(size)}
            </div>
            {closeField()}
        </XlReactFormItemControl>;
    }
}
