import PropTypes from 'prop-types';

export const tableLayoutType = PropTypes.arrayOf(PropTypes.shape({
    dataKey: PropTypes.string.isRequired,
    sortKey: PropTypes.string.isRequired,
    title: PropTypes.string.isRequired
}));

export const tableFilterType = PropTypes.objectOf(PropTypes.string.isRequired);

export const tableSortingType = PropTypes.shape({
    name: PropTypes.string,
    order: PropTypes.string
});

export const tablePagingType = PropTypes.shape({
    nextPageText: PropTypes.string,
    page: PropTypes.number.isRequired,
    pageSizes: PropTypes.arrayOf(PropTypes.shape({
        text: PropTypes.string.isRequired,
        value: PropTypes.number.isRequired
    })).isRequired,
    pageStartIndex: PropTypes.number,
    paginationPosition: PropTypes.string,
    paginationSize: PropTypes.number,
    prePageText: PropTypes.string,
    resultsPerPage: PropTypes.number.isRequired
});

export const filterSectionPositionType = PropTypes.shape({
    column: PropTypes.number.isRequired,
    row: PropTypes.node.isRequired
});

export const filterSectionBaseType = {
    className: PropTypes.string,
    label: PropTypes.string.isRequired,
    position: filterSectionPositionType.isRequired
};

export const filterSectionInputType = PropTypes.shape({
    ...filterSectionBaseType,
    fieldPath: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    placeholder: PropTypes.string,
    type: PropTypes.oneOf(['input']).isRequired
});

export const filterSectionAutocompleteType = PropTypes.shape({
    ...filterSectionBaseType,
    placeholder: PropTypes.string,
    type: PropTypes.oneOf(['autocomplete']).isRequired
});

export const filterSectionTagType = PropTypes.shape({
    ...filterSectionBaseType,
    type: PropTypes.oneOf(['tag']).isRequired
});

export const filterSectionCustomType = PropTypes.shape({
    component: PropTypes.element.isRequired,
    position: filterSectionPositionType.isRequired,
    type: PropTypes.oneOf(['custom']).isRequired
});

export const tableFiltersType = PropTypes.arrayOf(
    PropTypes.oneOfType([
        filterSectionInputType,
        filterSectionAutocompleteType,
        filterSectionTagType,
        filterSectionCustomType
    ])
);
