import {Action, Expectation} from 'protractor-base-dsl';
import WidgetAutocomplete from '../widget/widget-autocomplete';

const itemSelector = (itemNumber) =>
    `.testComponent .xl-react-dip-list-of-ci .xl-scrollable-panel .xl-list-row:nth-child(${itemNumber})`;

export const clearFilter = () => {
    const filterClearSelector = '.testComponent .xl-react-widget-filter .xl-react-btn-close';
    return Action.click(filterClearSelector);
};

export const expectFilterEnabled = () =>
    Expectation.enabled('.testComponent .xl-react-widget-filter');

export const expectSelectItemNotPresent = (itemNumber) =>
    Expectation.notPresent(itemSelector(itemNumber));

export const expectSelectedItemToEqual = (itemNumber, value) =>
    Expectation.attributeContainsValue(itemSelector(itemNumber), 'data-id', value);

export const expectClearIconOfItemToBePresent = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Expectation.present(closeSymbolSelector);
};

export const expectClearIconOfItemToBeNotPresent = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Expectation.notPresent(closeSymbolSelector);
};

export const removeSelectedItem = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Action.click(closeSymbolSelector);
};

export const setFilter = (value) => {
    const filterSelector = '.testComponent .xl-react-widget-filter input';
    return Action.typeText(filterSelector, value);
};

const DipListOfCi = {
    ...WidgetAutocomplete,
    clearFilter,
    expectFilterEnabled,
    expectSelectItemNotPresent,
    expectSelectedItemToEqual,
    expectClearIconOfItemToBePresent,
    expectClearIconOfItemToBeNotPresent,
    removeSelectedItem,
    setFilter
};

export default DipListOfCi;

global.DipListOfCi = DipListOfCi;
