import {Action, Expectation} from 'protractor-base-dsl';
import WidgetAutocomplete from '../widget/widget-autocomplete';

const itemSelector = (itemNumber) =>
    `.testComponent .xl-react-dip-set-of-ci .xl-scrollable-panel .xl-list-row:nth-child(${itemNumber})`;

export const clearFilter = () => {
    const filterClearSelector = '.testComponent .xl-react-widget-filter .xl-react-btn-close';
    return Action.click(filterClearSelector);
};

export const expectFilterEnabled = () =>
    Expectation.enabled('.testComponent .xl-react-widget-filter');

export const expectSelectedItemEnabled = (itemNumber) =>
    Expectation.enabled(itemSelector(itemNumber));

export const expectSelectedItemToEqual = (itemNumber, value) =>
    Expectation.attributeContainsValue(itemSelector(itemNumber), 'data-id', value);

export const expectSelectedItemNotPresent = (itemNumber) =>
    Expectation.notPresent(itemSelector(itemNumber));

export const expectClearIconOfItemToBePresent = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Expectation.present(closeSymbolSelector);
};

export const expectClearIconOfItemToBeNotPresent = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Expectation.notPresent(closeSymbolSelector);
};

export const expectComponentDisabled = () => {
    WidgetAutocomplete.expectDisabled();
    expectFilterEnabled();
    return expectClearIconOfItemToBeNotPresent(1);
};

export const expectComponentEnabled = () => {
    WidgetAutocomplete.expectEnabled();
    expectFilterEnabled();
    return expectClearIconOfItemToBePresent(1);
};

export const removeSelectedItem = (itemNumber) => {
    const closeSymbolSelector = `${itemSelector(itemNumber)} .xl-react-btn-close`;
    return Action.click(closeSymbolSelector);
};

export const setFilter = (value) => {
    const filterSelector = '.testComponent .xl-react-widget-filter input';
    return Action.typeText(filterSelector, value);
};

const DipSetOfCi = {
    ...WidgetAutocomplete,
    clearFilter,
    expectClearIconOfItemToBeNotPresent,
    expectFilterEnabled,
    expectClearIconOfItemToBePresent,
    expectComponentDisabled,
    expectComponentEnabled,
    expectSelectedItemEnabled,
    expectSelectedItemNotPresent,
    expectSelectedItemToEqual,
    removeSelectedItem,
    setFilter
};
export default DipSetOfCi;
global.DipSetOfCi = DipSetOfCi;
