import R from 'ramda';
import {Action, ActionUtil, Expectation, WaitCondition} from 'protractor-base-dsl';
import XlComponentsHelper from '../../common/xl-components-helper';
import WidgetTable from './widget-table';

const tableSelector = '.testComponent .xl-react-widget-redux-table';
const filterPanelSelector = `${tableSelector} .xl-react-redux-table-filters-panel`;

const expectAboveTableFilterPanel = (filterLabels) => {
    const selector = `${filterPanelSelector} .xl-react-components-label label`;
    element.all(By.css(selector)).getText().then((labels) => {
        if (!R.equals(filterLabels, labels)) {
            throw new Error(`Expected filters are [${filterLabels}], actual: [${labels}]`);
        }
    });
};

const expectColumnTableFilter = (columnNames) => {
    const check = (columnName) => Expectation.displayed(`${tableSelector} .xl-react-table-${columnName}`);
    R.forEach(check, columnNames);
};

const changeFilterType = (filterType) => {
    const selector = `${tableSelector} .type-of-filter`;

    const clearSelection = () => {
        const action = () => Action.click(`${selector} .xl-react-btn-close`);
        const condition = () => WaitCondition.textEquals(`${selector} input`, '');
        ActionUtil.repeatAction(action, condition);
    };

    const setValue = (value) => {
        const action = () => Action.typeText(`${selector} input`, value);
        const condition = () => WaitCondition.textEquals(`${selector} input`, value);
        return ActionUtil.repeatAction(action, condition);
    };

    Expectation.displayed(selector);
    clearSelection();
    setValue(filterType);
};

const columnFilterBy = (columnName, value) => {
    Action.typeNewText(`${tableSelector} .xl-react-table-${columnName} input`, value);
};

const aboveTableFilterBy = (filterLabel, value = '') => {
    const parent = element(By.css(`${tableSelector}`));

    ActionUtil.expectExecutedAction(() => XlComponentsHelper.inputByLabel(filterLabel, parent)
        .clear()
        .sendKeys(value));
};

const clearAboveTableFilters = (filterLabels) => {
    const parent = element(By.css(`${tableSelector}`));
    R.forEach((filterLabel) => {
        XlComponentsHelper.clearInputByLabel(filterLabel, parent);
    }, filterLabels);
};

const clearColumnFilters = (filterNames) => {
    R.forEach((filterName) => {
        Action.clearText(`${tableSelector} .xl-react-table-${filterName} input`);
    }, filterNames);
};

const WidgetReduxTable = {
    ...WidgetTable,
    aboveTableFilterBy,
    changeFilterType,
    clearAboveTableFilters,
    clearColumnFilters,
    columnFilterBy,
    expectAboveTableFilterPanel,
    expectColumnTableFilter
};
export default WidgetReduxTable;
global.WidgetReduxTable = WidgetReduxTable;
