import React from 'react';
import {mount} from 'enzyme';

import {XlReactDipBoolean} from '~/app/xl-react-dip/xl-react-dip-boolean/xl-react-dip-boolean';
import Helper from '../../validations-spec-helper';
import {ERROR} from '~/app/utils/ci-utils';

describe('XlReactDipBoolean', () => {
    it('boolean component without label', () => {
        const wrapper = mount(<XlReactDipBoolean/>);
        expect(wrapper.find('XlReactWidgetLabel')).not.toExist();
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
    });

    it('boolean component with label', () => {
        const wrapper = mount(<XlReactDipBoolean metadata={{label: "I'm boolean"}}/>);
        const label = wrapper.find('XlReactWidgetLabel');
        expect(label).toExist();
        expect(label.prop('metadata').label).toEqual("I'm boolean");
        expect(wrapper.find('.xl-react-components-input')).toExist();
    });

    it('disabled boolean component', () => {
        const wrapper = mount(<XlReactDipBoolean displayMode="visual"/>);
        expect(wrapper.find('input[type="checkbox"][disabled]')).toExist();
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
    });

    it('boolean component with default value', () => {
        const wrapper = mount(<XlReactDipBoolean metadata={{default: true}}/>);
        expect(wrapper.find('input[checked]')).toExist();
    });

    it('should honour custom validators', () => {
        const validator = (model) => ({
            condition: !model,
            message: {show: false, text: 'You must agree to the license terms!', type: ERROR}
        });
        const element = <XlReactDipBoolean model={true} validators={[validator]}/>;
        const {changeInput, errMsg} = new Helper('.xl-react-dip-boolean').buildFrom(element);
        changeInput(false);
        expect(errMsg()).toHaveText('You must agree to the license terms!');
    });
});
