import {prepareViewModel, addItem, moveItem} from '~/app/xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service'; // eslint-disable-line max-len

describe('Xl Dip Collection of CI Service', () => {
    describe('prepareViewModel', () => {
        const items = [
            {id: 1, label: 'First item'},
            {id: 2, label: 'Second Item'},
            {id: 3, label: 'Third'}
        ];

        it('should properly prepare non-filtered view model', () => {
            expect(prepareViewModel(items, '', 'label')).toEqual([['First item', 0], ['Second Item', 1], ['Third', 2]]);
        });

        it('should properly prepare non-filtered view model', () => {
            const expectedResult = [['First item', 0], ['Second Item', 1]];
            expect(prepareViewModel(items, 'iTem', 'label')).toEqual(expectedResult);
            expect(prepareViewModel(items, 'ITEM', 'label')).toEqual(expectedResult);
        });
    });

    describe('addItem', () => {
        const input = [{id: 1, label: 'A'}];
        it('should add items (duplicates allowed when unique: false)', () => {
            expect(addItem(input, {id: 2, label: 'B'}, false)).toEqual([{id: 1, label: 'A'}, {id: 2, label: 'B'}]);
            expect(addItem(input, {id: 1, label: 'A'}, false)).toEqual([{id: 1, label: 'A'}, {id: 1, label: 'A'}]);
        });

        it('should add items (duplicates are NOT allowed when unique: true)', () => {
            expect(addItem(input, {id: 2, label: 'B'}, true)).toEqual([{id: 1, label: 'A'}, {id: 2, label: 'B'}]);
            expect(addItem(input, {id: 1, label: 'A'}, true)).toEqual([{id: 1, label: 'A'}]);
        });
    });

    describe('moveItem', () => {
        it('should move item within an array', () => {
            const input = [
                {id: 1, label: 'A'},
                {id: 2, label: 'B'},
                {id: 3, label: 'C'}
            ];
            expect(moveItem(2, 0, input)).toEqual([
                {id: 3, label: 'C'},
                {id: 1, label: 'A'},
                {id: 2, label: 'B'}
            ]);
            expect(moveItem(1, 2, input)).toEqual([
                {id: 1, label: 'A'},
                {id: 3, label: 'C'},
                {id: 2, label: 'B'}
            ]);
        });
    });
});
