import R from 'ramda';
import {sortItems} from '~/app/xl-react-dip/xl-react-dip-set-of-ci/dip-set-of-cis-utils';

describe('dip-set-of-cis-utils', () => {
    describe('sortItems', () => {
        it('should not fail on undefined or empty input', () => {
            expect(sortItems([])).toEqual([]);
            expect(sortItems(undefined)).toEqual(undefined);
        });
        it('should sort CI`s alphabetically with ignoring the name of root node', () => {
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];
            const expected = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            expect(sortItems(input)).toEqual(expected);
        });
        it('should sort objects alphabetically with ignoring the name of root node', () => {
            const transformToObject = R.map((item) => ({id: item, type: 'someType'}));
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];

            const sortedOutput = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            const expected = transformToObject(sortedOutput);
            expect(sortItems(transformToObject(input))).toEqual(expected);
        });
        it('should sort objects (with custom objectLabel) alphabetically with ignoring the name of root node', () => {
            const transformToObject = R.map((item) => ({name: item, type: 'someType'}));
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];

            const sortedOutput = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            const expected = transformToObject(sortedOutput);
            expect(sortItems(transformToObject(input), 'name')).toEqual(expected);
        });
    });
});
